const {
  JuphoonWebRTCGuest,
  StompSocketAuthType,
  ConferenceEventType,
  GuestEventType,
  SimpleRoleCompose,
  SimpleSVCResolution, StorageManagerInstance, WebMediaDevicesInstance, IRTCCoreEventType, PictureSize, GuestCallState,
  ConferenceClientState,
  LoggerLevel,
  ReporterManagerInstance
} = JRTCSDK;

const sdkVersionInfo = JuphoonWebRTCGuest.getVersion();
document.querySelector('.version-info').textContent = sdkVersionInfo.version + '-' + sdkVersionInfo.gitHashShort + '-' + sdkVersionInfo.buildTime;

const app = Vue.createApp({
  data() {
    return {
      StompSocketAuthType: StompSocketAuthType,
      PictureSize: PictureSize,
      LoggerLevel: LoggerLevel,
      environmentSettings: {},
      conferenceSettings: {},
      performanceOptions: {},
      clientList: [],
      fileVideoSource: undefined,
      prevRenderClient: undefined
    };
  },
  created() {
    WebMediaDevicesInstance.refresh()
      .then(() => this.WebMediaDevicesInstance = WebMediaDevicesInstance);

    this.reset();
    const settingFormData = StorageManagerInstance.get('settingFormData_webrtc-guest-sdk');

    if (settingFormData) {
      Object.entries(settingFormData)
        .forEach((entity) => this[entity[0]] = entity[1]);
    }

    window.onbeforeunload = (ev) => {
      const {environmentSettings, conferenceSettings, performanceOptions} = this;
      StorageManagerInstance.set('settingFormData_webrtc-guest-sdk', {
        environmentSettings,
        conferenceSettings,
        performanceOptions
      });

      ev.preventDefault();
      ev.returnValue = '';
      return '';
    };
  },
  unmounted() {
  },
  methods: {
    reset() {
      this.environmentSettings = {
        environmentConfig: {
          address: 'https://videobu.juphoon.cloud:10047',
          appKey: '05c97ce0d0a84d2592334093',
          token: '',
          authType: StompSocketAuthType.REDIS
        },
      };
      this.conferenceSettings = {
        videoSize: PictureSize.MIN,
        svcConfig: SimpleSVCResolution.HD,
        groupNumber: '10086',
        token: '10000',
      };
      this.performanceOptions = {
        clientCount: 1,
        clientCreateInterval: 10,
        talkTime: 30,
        loopCount: 1,
        useVideoSource: false,
      };
    },
    start() {
      JuphoonWebRTCGuest.setConfig(this.environmentSettings.environmentConfig);
      const promises = [];

      for (let i = 0; i < this.performanceOptions.clientCount; i++) {
        const promise = this.prepareClient()
          .then((client) => this.clientList.push(client));

        promises.push(promise);
      }

      Promise.all(promises)
        .then(() => new Promise((resolve, reject) => {
          if (this.performanceOptions.useVideoSource) {
            document.getElementById('video-source').play();
            const dom = document.getElementById('video-source');
            if (dom.captureStream) {
              this.fileVideoSource = document.getElementById('video-source').captureStream();
            } else if (dom.mozCaptureStream) {
              this.fileVideoSource = document.getElementById('video-source').mozCaptureStream();
            }

            if (!this.conferenceSettings.video && this.fileVideoSource) {
              this.fileVideoSource = new MediaStream(this.fileVideoSource.getAudioTracks());
            }

            resolve();
          } else {
            WebMediaDevicesInstance.requestUserMedia({
              audio: true,
              video: this.conferenceSettings.video
            })
              .then((stream) => this.fileVideoSource = stream)
              .then(resolve)
              .catch(reject);
          }
        }))
        .then(() => {
          const resultPromises = [];
          for (let i = 0; i < this.clientList.length; i++) {
            resultPromises.push(new Promise((resolve, reject) => {
              setTimeout(() => {
                this.loopBusinessFlow(this.clientList[i], i).then(resolve).catch(reject);
              }, this.performanceOptions.clientCreateInterval * i * 1e3);
            }));
          }

          return Promise.all(resultPromises);
        })
        .then(() => {
          alert('测试结束');
        })
        .catch((reason) => {
          console.error(reason);
          alert(reason.message);
        })
        .finally(() => {
          this.stop();
        });
    },
    stop() {
      const temp = this.clientList.slice();

      temp.forEach((client, index) => {
        clearTimeout(client.__talkTimeoutKey);
        client.logout()
          .then(() => this.clientList.splice(index, 1));
      });
      WebMediaDevicesInstance.releaseUserMedia();
      this.clientList = [];
    },
    prepareClient() {
      return JuphoonWebRTCGuest.getToken()
        .then((_token) => {
          const client = JuphoonWebRTCGuest.createGuestClient(this.environmentSettings.environmentConfig.appKey, _token);

          client.addEventListeners(
            [
              ConferenceEventType.CONFERENCE_PARTICIPANT_JOIN,
              ConferenceEventType.CONFERENCE_PARTICIPANT_LEFT,
              ConferenceEventType.CONFERENCE_PARTICIPANT_UPDATE,
              ConferenceEventType.CLIENT_STATE_CHANGE,
              GuestEventType.CALL_STATE_CHANGE,
            ],
            () => this.$forceUpdate()
          );

          client.remainCount = this.performanceOptions.loopCount;
          return client;
        });
    },
    loopBusinessFlow(client, index) {
      let promise = client.login();

      for (let i = 0; i < this.performanceOptions.loopCount; i++) {
        promise = promise.then(() => this.startBusinessFlow(client, index))
          .then(() => client.remainCount--);
      }

      return promise;
    },
    startBusinessFlow(client, index) {
      return client.call(this.conferenceSettings.groupNumber, {
        svcResolution: this.conferenceSettings.svcConfig,
        video: this.conferenceSettings.video,
      })
        .then(() => client.publishMediaStream(this.fileVideoSource))
        .then(() => client.startVideo())
        .then(() => new Promise((resolve, reject) => {
          client.core.addEventListener(IRTCCoreEventType.STATS, (ev) => client.statsInfo = ev.message.data.stats);
          client.core.getStats();

          client.addEventListener(GuestEventType.CALL_STATE_CHANGE, (ev, removeSelf) => {
            const {callState} = ev.message.data;

            switch (callState) {
              case GuestCallState.TALKING:
                client.__talkTimeoutKey = setTimeout(() => {
                  client.term().then(resolve).catch(reject);

                }, this.performanceOptions.talkTime * 1e3);

                const agent = Object.values(client.participantMap).find((p) => p.isAgent());

                if (agent) client.requestVideo(agent, this.conferenceSettings.videoSize);
                break;
              case GuestCallState.TERMED:
                resolve();
                removeSelf();
                break;
            }
          })
        }));
    },
    renderVideo(client, index) {
      new Promise((resolve, reject) => {
        if (this.prevRenderClient) {
          Promise.all([
            this.prevRenderClient.remoteStream && this.prevRenderClient.remoteStream.stop(),
            this.prevRenderClient.localStream && this.prevRenderClient.localStream.stop()
          ])
            .then(resolve)
            .catch(reject);
        } else {
          resolve();
        }
      })
        .then(() => {
          client.remoteStream.start('remote-stream', document)
            .catch((reason) => console.error(reason));
          client.localStream && client.localStream.start('local-stream', document)
            .catch((reason) => console.error(reason));

          this.prevRenderClient = client;
        });
    },
    loggerLevelChange() {
      ReporterManagerInstance.setLogLevel(this.environmentSettings.logLevel);
    },
    openFAQPage(anchor) {
      window.open(`./FAQ.html#${anchor}`);
    },
    downloadLog() {
      const blob = ReporterManagerInstance.asTextBlob(),
        blobUrl = window.URL.createObjectURL(blob),
        tempLink = document.createElement('a');

      tempLink.style.display = 'none';
      tempLink.href = blobUrl;
      tempLink.setAttribute('download', `${window.location.host}-${Date.now()}.log`);

      if (typeof tempLink.download === 'undefined') {
        tempLink.setAttribute('target', '_blank');
      }

      document.body.appendChild(tempLink);
      tempLink.click();
      document.body.removeChild(tempLink);

      setTimeout(() => {
        window.URL.revokeObjectURL(blobUrl);
      });
    }
  }
})
  .mount('#app');
